/*** Scramble V 1.0. (C) 1999/2000 by S. Krahmer. GPL.
 *** So you use it at your own risk!
 *** You can fetch the license-file 'COPYING' from my homepage
 *** http://www.cs.uni-potsdam.de/homepages/students/linuxer , package
 *** 'logit'.
 ***/

%{
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#define PROTOFILE "/etc/ciffer.proto"

/* which line */
int yyline = 1;

/* how much strings we must decrypt at runtime */
int how_much = 0;

/* the strings itself */
char *strings_to_decrypt[1000] = {0};

/* the C-file */
FILE *cout;

int yyerror(const char*);
int yylex(void);

int scramble(char *, char*);
char *my_encrypt(char *s);
%}

/* we don't need a 'int' at all,
 * but who cares..
 */
%union {
	int i;
	char s[1001];
}

/* some terminals
 */
%token <i> CHAR 
%token <s> IDENT
%token <s> STRING

%type <s> strings
%type <s> str

%start strings

%%

/* what we accept is a list of strings */

strings:	str
	    |	strings str		/* use left recursion, it's faster! */
	    ;

str:	 	CHAR IDENT '[' ']' '=' STRING ';' { scramble($2, $6); }
	    ;
%%

int
yyerror(const char *s)
{
	fprintf(stderr, "%s (at line %d)\n", s, yyline);
	return -1;
}

int
main(int argc, char **argv)
{	
	int i = 0;
	int fd = 0;
	char buf[1001] = {0};
		
	if (argc < 3) {
		printf("usage: %s [C-file] [proto-file]\n\n", argv[0]);
		exit(1);
	}
	
	/* dirty hack */
	close(0); close(1);
	
	/* should give stdin */
	if (fopen(argv[2], "r") == NULL) {
		perror("fopen");
		exit(errno);
	}
	/* should give stdout */
	if (fopen("scramble.out.c", "w+") == NULL) {
		perror("fopen");
		exit(errno);
	}
	if ((cout = fopen(argv[1], "r")) == NULL) {
		perror("fopen");
		exit(errno);
	}
	/* to scramble.out.c */
	printf("/*** Automatically generated C-file by scramble V 1.0\n"
	       " *** Do not edit !\n"
	       " ***/\n\n");	
	
	/* look for strings */
	yyparse();

	printf("void deciffer(char *);\n\n");	
	printf("static void decrypt_at_runtime() __attribute__ ((constructor));\n\n");
	printf("static void\ndecrypt_at_runtime()\n{\n");
	
	while (i < how_much) {
		printf("\tdeciffer(%s);\n", strings_to_decrypt[i]);
		i++;
	}
	printf("}\n\n");
	
	/* append c-file */
	while (fgets(buf, 1000, cout)) {
		/* skip '#include "proto.h"' b/c we write the string into the file */
		if (!strstr(buf, "include") || !strstr(buf, argv[2]))
			printf("%s", buf);
		memset(buf, 0, 1000);
	}
	printf("\n");		
	
	if ((fd = open(PROTOFILE, O_RDONLY)) < 0) {
		perror("open (ciffer.proto)");
		exit(errno);
	}
	/* append ciffer.proto to file */
	while (read(fd, buf, 1000) > 0) {
		printf("%s", buf);
		memset(buf, 0, 1000);
	}
		
	fclose(cout);
	close(0);
	return 0;
}

int 
scramble(char *ident, char *string)
{
	char *s1;
	
	/* skip last '"' */
	string[strlen(string)-1] = '\0';

	s1 = my_encrypt(string+1);		/* step over '"' */
	
	printf("char %s[] = \"%s\";\n", ident, s1);
	
	/* we must know later, wich strings at runtime we have to decrypt */
	if ((strings_to_decrypt[how_much] = (char*)malloc(sizeof(ident))) == NULL) {
		perror("malloc");
		exit(errno);
	}
	memset(strings_to_decrypt[how_much], 0, sizeof(ident));
	
	strcpy(strings_to_decrypt[how_much], ident);
	fprintf(stderr, "scramble '%s'\n", strings_to_decrypt[how_much]);
	how_much++;
	
	free(s1);
	return 0;
}

char *
my_encrypt(char *s)
{
	char *es = NULL;
	int i = 0;
	static char key[10] = {0};
	static int first = 0;
	
	if ((es = malloc(2*strlen(s))) == NULL) {
		perror("malloc");
		exit(errno);
	}
	memset(es, 0, 2*strlen(s));
	
	/* generate a key - only once ! */
	if (first == 0) {
		fprintf(stderr, "Your tag is:");
		srand(time(NULL));
		for (i = 0; i < 10; i++) 
			key[i] = 1 + (int)(255.0 * rand()/(RAND_MAX + 1.0));
		printf("const char _scramble10_key_[] = \"");
		for (i = 0; i < 10; i++) {
			/* print key to file */
			printf("%02x", (unsigned char)key[i]);
			/* give out tag */
			fprintf(stderr, "%02x", (unsigned char)key[i]);
		}
		printf("\";\n");
		fprintf(stderr, "\n");
#ifdef DEBUG
		fprintf(stderr, "%s\n", key);
#endif
	} /* if */
	first = 1;
	for (i = 0; i < strlen(s); i++) 
		sprintf(es + 2*i, "%02x", (unsigned char)(s[i] ^ key[i % 10]));
	return es;
}
